# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"get_unicode": "6.2_desc.metrics.java.ipynb",
         "simulate_getting_dataframes_from_mongo": "6.2_desc.metrics.java.ipynb",
         "add_mccabe_metrics": "6.2_desc.metrics.java.ipynb",
         "create_parser_builds": "6.2_desc.metrics.java.ipynb",
         "lang_keywords": "6.2_desc.metrics.java.ipynb",
         "find_class_nodes": "6.2_desc.metrics.java.ipynb",
         "find_method_nodes": "6.2_desc.metrics.java.ipynb",
         "find_field_names": "6.2_desc.metrics.java.ipynb",
         "find_string_in_text": "6.2_desc.metrics.java.ipynb",
         "distinct_field_calls": "6.2_desc.metrics.java.ipynb",
         "calculate_lcom5": "6.2_desc.metrics.java.ipynb",
         "add_lcom5": "6.2_desc.metrics.java.ipynb",
         "flatten_lol": "6.2_desc.metrics.java.ipynb",
         "display_numeric_col_stats": "6.2_desc.metrics.java.ipynb",
         "display_numeric_col_hist": "6.2_desc.metrics.java.ipynb"}

modules = ["desc/metrics/java.py"].py",
           "exp/info-[inspect].py"].py"]

doc_url = "https://ncoop57.github.io/ds4se/"

git_url = "https://github.com/ncoop57/ds4se/tree/master/"

def custom_doc_links(name): return None
