# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/0.6_mgmnt.prep.nltk.ipynb (unless otherwise specified).

__all__ = ['get_nltk_docs']

# Cell
# Imports
import pandas as pd
import random
import sentencepiece as sp

from .traceability import *
from fastprogress.fastprogress import master_bar
from pathlib import Path

# Cell
def get_nltk_docs(df, src_fns, trgt_fns):
    src_docs, trgt_docs = [], []

    for src, trgt in zip(src_fns, trgt_fns):
        src_docs.append(df.loc[df.file == src].text.values[0])
        trgt_docs.append(df.loc[df.file == trgt].text.values[0])

    return src_docs, trgt_docs