# AUTOGENERATED! DO NOT EDIT! File to edit: nbs/2.8_repr.sentence_transformers.ipynb (unless otherwise specified).

__all__ = []

# Cell
# Imports
import numpy as np
from gensim import corpora

# Cell
# Imports
import gensim
import pandas as pd
from itertools import product
from random import sample
import functools
import os